/*---------------------------------------------------------------
 *	周期ハンドラを使ったブザー制御(μT-Kernel 3.0用)
 *
 *	Copyright (C) 2022-2023 by T3 WG of TRON Forum
 *---------------------------------------------------------------*/
#include <tk/tkernel.h>
#include <tm/tmonitor.h>

//-------- ログの記録機能 ---------------------------------------
#define LONG_MIN 0x80000000		// 整数の最小値を特殊な意味で使用
#define NO_PARAM LONG_MIN		// ログのパラメータが無い場合の指定

// ログを記録するための配列
#define	MAX_LOG	2000
UB	*log_msg[MAX_LOG];
W	log_param[MAX_LOG];
UW	log_time[MAX_LOG];
W	log_index = 0;

// ログを記録するための関数
void log(UB *msg, W param)
{
	SYSTIM	tim;
	tk_get_tim(&tim);				// 現在時刻を取得
	log_msg[log_index] = msg;		// ログメッセージを保存
	log_param[log_index] = param;	// ログメッセージのパラメータを保存
	log_time[log_index] = tim.lo;	// ミリ秒単位の時刻の下位32ビットを保存
	if(++log_index >= MAX_LOG){		// ログ記録用の配列のインデクスを更新
		tm_printf((UB*) "*** log overflow !\n");
		log_index--;
	}
}

// 最後にまとめてログを表示する関数
void print_log(void)
{
	UB	msg_param[16];
	W	cnt, dtime;

	for(cnt = 0 ; cnt < log_index ; cnt++){

		/* usermain実行開始時からの経過時間(相対時刻)を計算 */
		dtime = log_time[cnt] - log_time[0];

		tm_sprintf(msg_param, "");			// ログにパラメータが無い場合
		if(log_param[cnt] != NO_PARAM){		// パラメータ(整数)がある場合
			tm_sprintf(msg_param, "%d", log_param[cnt]);
		}
		tm_printf((UB*) "%4d:%6dms: %s%s\n",
			(cnt + 1), dtime, log_msg[cnt], msg_param);
	}
}

//-------- GPIO関連の共通処理 -----------------------------------
// GPIOの特定のピンpinの出力をval(0または1)に設定
//	pinで指定された以外のピンの状態は変化しない
LOCAL void out_gpio_pin(UW port, UW pin, UW val)
{
	INT port_addr;

	if(port == 1){							// P1の場合
		if(val)
			port_addr = GPIO(P1, OUTSET);	// P1でvalが1の場合
		else							
			port_addr = GPIO(P1, OUTCLR);	// P1でvalが0の場合

	} else {								// P0の場合
		if(val)
			port_addr = GPIO(P0, OUTSET);	// P0でvalが1の場合
		else
			port_addr = GPIO(P0, OUTCLR);	// P0でvalが0の場合
	}
	out_w(port_addr, (1 << pin));			// 指定のピンに1を出力
}

//-------- スピーカーへの出力 -----------------------------------
LOCAL BOOL	speaker_val = FALSE;		// 現在のスピーカーへの出力値

// スピーカーへの出力値の0と1を反転する関数(※F)
void speaker_onoff(void)
{
	if(speaker_val)						// speaker_valの値に応じてログを記録
		log("speaker_on", NO_PARAM);	// スピーカーへの出力が0から1に変化
	else
		log("speaker_off", NO_PARAM);	// スピーカーへの出力が1から0に変化

	// スピーカーに接続されたGPIOピン(P0.00)にspeaker_valの値を出力
	out_gpio_pin(0, 0, speaker_val);	// port=0, pin=0を指定(※G)

	// 次の出力のために値を反転(※J)
	speaker_val = (! speaker_val);
}

//-------- 周期ハンドラによるスピーカーの出力制御 ---------------
const INT half_cycle_ms = 1;		// 音の周期時間の半分(ミリ秒単位)(※N)

// 周期ハンドラSの関連情報(※L)
const T_CCYC ccycS = {0, TA_HLNG, &speaker_onoff, half_cycle_ms, 0};
ID cycidS;							// 周期ハンドラSのID

EXPORT void usermain(void)
{
	// プログラムの名称とティック時間などの情報をログに記録
	log("List5: Buzzer using Cyclic Handler", NO_PARAM);
	log("CNF_TIMER_PERIOD=", CNF_TIMER_PERIOD);
	log("half_cycle_ms=", half_cycle_ms);
	log("Start User-main program", NO_PARAM);

    // スピーカーに接続されたGPIOピン(P0.00)を出力に設定
    out_w(GPIO(P0, PIN_CNF(0)), (1 << 0));

	cycidS = tk_cre_cyc(&ccycS);	// 周期ハンドラSの生成
	tk_sta_cyc(cycidS);				// 周期ハンドラSの動作開始(※M)
	tk_dly_tsk(1000);				// 1秒間の時間待ち
	tk_stp_cyc(cycidS);				// 周期ハンドラSの動作停止

	log("End User-main program", NO_PARAM);
	print_log();					// ログを表示
	tk_slp_tsk(TMO_FEVR);			// 永久待ち
}
